@@ -47,12 +47,12 @@ import java.io.Serializable;
  *  When Envelope objects are created or initialized,
  *  the supplies extent values are automatically sorted into the correct order.
  *
- *@version 1.6
+ *@version 1.7
  */
 public class Envelope
     implements Serializable
 {
-    private static final long serialVersionUID = 5873921885273102420L;    
+    private static final long serialVersionUID = 5873921885273102420L;
 
     public int hashCode() {
         //Algorithm from Effective Java by Joshua Bloch [Jon Aquino]
@@ -356,7 +356,38 @@ public class Envelope
     expandToInclude(p.x, p.y);
   }
 
-  //<<TODO:FEATURE>> #expandBy(double distance) [Jon Aquino]
+  /**
+   * Expands this envelope by a given distance in all directions.
+   * Both positive and negative distances are supported.
+   *
+   * @param distance the distance to expand the envelope
+   * @return this envelope
+   */
+  public void expandBy(double distance)
+  {
+    expandBy(distance, distance);
+  }
+
+  /**
+   * Expands this envelope by a given distance in all directions.
+   * Both positive and negative distances are supported.
+   *
+   * @param deltaX the distance to expand the envelope along the the X axis
+   * @param deltaY the distance to expand the envelope along the the Y axis
+   */
+  public void expandBy(double deltaX, double deltaY)
+  {
+    if (isNull()) return;
+
+    minx -= deltaX;
+    maxx += deltaX;
+    miny -= deltaY;
+    maxy += deltaY;
+
+    // check for envelope disappearing
+    if (minx > maxx || miny > maxy)
+      setToNull();
+  }
 
   /**
    *  Enlarges the boundary of the <code>Envelope</code> so that it contains
@@ -421,6 +452,52 @@ public class Envelope
     }
   }
 
+  /**
+   * Translates this envelope by given amounts in the X and Y direction.
+   *
+   * @param transX the amount to translate along the X axis
+   * @param transY the amount to translate along the Y axis
+   */
+  public void translate(double transX, double transY) {
+    if (isNull()) {
+      return;
+    }
+    init(getMinX() + transX, getMaxX() + transX,
+         getMinY() + transY, getMaxY() + transY);
+  }
+
+  /**
+   * Computes the coordinate of the centre of this envelope (as long as it is non-null
+   *
+   * @return the centre coordinate of this envelope
+   * <code>null</code> if the envelope is null
+   */
+  public Coordinate centre() {
+    if (isNull()) return null;
+    return new Coordinate(
+        (getMinX() + getMaxX()) / 2.0,
+        (getMinY() + getMaxY()) / 2.0);
+  }
+
+  /**
+   * Computes the intersection of two {@link Envelopes}
+   *
+   * @param env the envelope to intersect with
+   * @return a new Envelope representing the intersection of the envelopes (this will be
+   * the null envelope if either argument is null, or they do not intersect
+   */
+  public Envelope intersection(Envelope env)
+  {
+    if (isNull() || env.isNull() || ! intersects(env)) return new Envelope();
+
+    double intMinX = minx > env.minx ? minx : env.minx;
+    double intMinY = miny > env.miny ? miny : env.miny;
+    double intMaxX = maxx < env.maxx ? maxx : env.maxx;
+    double intMaxY = maxy < env.maxy ? maxy : env.maxy;
+    return new Envelope(intMinX, intMaxX, intMinY, intMaxY);
+  }
+
+
   /**
    *  Returns <code>true</code> if the given point lies in or on the envelope.
    *
@@ -460,10 +537,10 @@ public class Envelope
    */
   public boolean intersects(Envelope other) {
       if (isNull() || other.isNull()) { return false; }
-    return !(other.getMinX() > maxx ||
-        other.getMaxX() < minx ||
-        other.getMinY() > maxy ||
-        other.getMaxY() < miny);
+    return !(other.minx > maxx ||
+        other.maxx < minx ||
+        other.miny > maxy ||
+        other.maxy < miny);
   }
   /**
    * @deprecated Use #intersects instead. In the future, #overlaps may be
@@ -561,7 +638,7 @@ public class Envelope
     return maxx == otherEnvelope.getMaxX() &&
         maxy == otherEnvelope.getMaxY() &&
         minx == otherEnvelope.getMinX() &&
-        maxx == otherEnvelope.getMaxX();
+        miny == otherEnvelope.getMinY();
   }
 
   public String toString()
