@@ -1,5 +1,3 @@
-
-
 /*
  * The JTS Topology Suite is a collection of Java classes that
  * implement the fundamental operations required to validate a given
@@ -49,9 +47,8 @@ import com.vividsolutions.jts.operation.relate.RelateOp;
 import com.vividsolutions.jts.operation.valid.IsValidOp;
 import com.vividsolutions.jts.util.Assert;
 /**
- *  Basic implementation of <code>Geometry</code>. <P>
- *
- *  <code>clone</code> returns a deep copy of the object.
+ * The base class for all geometric objects.
+ * <P>
  *
  *  <H3>Binary Predicates</H3>
  * Because it is not clear at this time
@@ -126,7 +123,7 @@ import com.vividsolutions.jts.util.Assert;
  *  topologically equal Geometries are added to HashMaps and HashSets, they
  *  remain distinct. This behaviour is desired in many cases.
  *
- *@version 1.6
+ *@version 1.7
  */
 public abstract class Geometry
     implements Cloneable, Comparable, Serializable
@@ -230,14 +227,12 @@ public abstract class Geometry
    *@return    the ID of the coordinate space in which the <code>Geometry</code>
    *      is defined.
    *
-   *  @deprecated use {@link getUserData} instead
    */
   public int getSRID() {
     return SRID;
   }
     /**
    *  Sets the ID of the Spatial Reference System used by the <code>Geometry</code>.
-   *  @deprecated use {@link setUserData} instead
    */
   public void setSRID(int SRID) {
     this.SRID = SRID;
@@ -337,20 +332,27 @@ public abstract class Geometry
   public abstract int getNumPoints();
 
   /**
-   *  Returns false if the <code>Geometry</code> not simple.
-   *  Subclasses provide their own definition of "simple". If
-   *  this <code>Geometry</code> is empty, returns <code>true</code>. <P>
-   *
-   *  In general, the SFS specifications of simplicity seem to follow the
-   *  following rule:
+   * Tests whether this {@link Geometry} is simple.
+   * In general, the SFS specification of simplicity
+   * follows the rule:
    *  <UL>
    *    <LI> A Geometry is simple iff the only self-intersections are at
    *    boundary points.
    *  </UL>
-   *  For all empty <code>Geometry</code>s, <code>isSimple</code> = <code>true</code>.
+   * Simplicity is defined for each {@link Geometry} subclass as follows:
+   * <ul>
+   * <li>Valid polygonal geometries are simple by definition, so
+   * <code>isSimple</code> trivially returns true.
+   * <li>Linear geometries are simple iff they do not self-intersect at points
+   * other than boundary points.
+   * <li>Zero-dimensional geometries (points) are simple iff they have no
+   * repeated points.
+   * <li>Empty <code>Geometry</code>s are always simple
+   * <ul>
    *
-   *@return    <code>true</code> if this <code>Geometry</code> has any points of
+   * @return    <code>true</code> if this <code>Geometry</code> has any points of
    *      self-tangency, self-intersection or other anomalous points
+   * @see #isValid
    */
   public abstract boolean isSimple();
 
@@ -576,24 +578,41 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is FF*FF****.
+   * Returns <code>true</code> if this geometry is disjoint to the specified geometry.
+   * <p>
+   * The <code>disjoint</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The two geometries have no point in common
+   * <li>The DE-9IM Intersection Matrix for the two geometries is FF*FF****
+   * <li>! <code>g.intersects(this)</code>
+   * (<code>disjoint</code> is the inverse of <code>intersects</code>)
+   * </ul>
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s are
    *      disjoint
+   *
+   * @see Geometry#intersects
    */
   public boolean disjoint(Geometry g) {
     return ! intersects(g);
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is FT*******, F**T***** or F***T****.
+   * Returns <code>true</code> if this geometry touches the
+   * specified geometry.
+   * <p>
+   * The <code>touches</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The geometries have at least one point in common, but their interiors do not intersect.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *   FT*******, F**T***** or F***T****
+   * </ul>
+   * If both geometries have dimension 0, this predicate returns <code>false</code>
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s touch;
-   *      Returns false if both <code>Geometry</code>s are points
+   *      Returns <code>false</code> if both <code>Geometry</code>s are points
    */
   public boolean touches(Geometry g) {
     // short-circuit test
@@ -603,16 +622,43 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if <code>disjoint</code> returns false.
+   * Returns <code>true</code> if this geometry intersects the specified geometry.
+   * <p>
+   * The <code>intersects</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The two geometries have at least one point in common
+   * <li>! <code>g.disjoint(this)</code>
+   * (<code>intersects</code> is the inverse of <code>disjoint</code>)
+   * </ul>
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s intersect
+   *
+   * @see Geometry#disjoint
    */
   public boolean intersects(Geometry g) {
+
     // short-circuit envelope test
     if (! getEnvelopeInternal().intersects(g.getEnvelopeInternal()))
       return false;
-    // optimizations for rectangle arguments
+
+    /**
+     * TODO: (MD) Add optimizations:
+     *
+     * - for P-A case:
+     * If P is in env(A), test for point-in-poly
+     *
+     * - for A-A case:
+     * If env(A1).overlaps(env(A2))
+     * test for overlaps via point-in-poly first (both ways)
+     * Possibly optimize selection of point to test by finding point of A1
+     * closest to centre of env(A2).
+     * (Is there a test where we shouldn't bother - e.g. if env A
+     * is much smaller than env B, maybe there's no point in testing
+     * pt(B) in env(A)?
+     */
+
+    // optimization for rectangle arguments
     if (isRectangle()) {
       return RectangleIntersects.intersects((Polygon) this, g);
     }
@@ -624,20 +670,27 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is
-   *  <UL>
-   *    <LI> T*T****** (for a point and a curve, a point and an area or a line
-   *    and an area)
-   *    <LI> 0******** (for two curves)
-   *  </UL>
-   *  .
+   * Returns <code>true</code> if this geometry crosses the
+   * specified geometry.
+   * <p>
+   * The <code>crosses</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The geometries have some but not all interior points in common.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *   <ul>
+   *    <li>T*T****** (for P/L, P/A, and L/A situations)
+   *    <li>T*****T** (for L/P, L/A, and A/L situations)
+   *    <li>0******** (for L/L situations)
+   *   </ul>
+   * </ul>
+   * For any other combination of dimensions this predicate returns <code>false</code>.
+   * <p>
+   * The SFS defined this predicate only for P/L, P/A, L/L, and L/A situations.
+   * JTS extends the definition to apply to L/P, A/P and A/L situations as well.
+   * This makes the relation symmetric.
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s cross.
-   *      For this function to return <code>true</code>, the <code>Geometry</code>
-   *      s must be a point and a curve; a point and a surface; two curves; or a
-   *      curve and a surface.
    */
   public boolean crosses(Geometry g) {
     // short-circuit test
@@ -647,30 +700,58 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is T*F**F***.
+   * Returns <code>true</code> if this geometry is within the
+   * specified geometry.
+   * <p>
+   * The <code>within</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>Every point of this geometry is a point of the other geometry,
+   * and the interiors of the two geometries have at least one point in common.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is T*F**F***
+   * <li><code>g.contains(this)</code>
+   * (<code>within</code> is the inverse of <code>contains</code>)
+   * </ul>
+   * An implication of the definition is that
+   * "The boundary of a Polygon is not within the Polygon".
+   * In other words, if a geometry G is a subset of
+   * the points in the boundary of a polygon P, <code>G.within(P) = false</code>
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if this <code>Geometry</code> is within
    *      <code>other</code>
+   *
+   * @see Geometry#contains
    */
   public boolean within(Geometry g) {
     return g.contains(this);
   }
 
   /**
-   *  Returns <code>true</code> if <code>other.within(this)</code> returns
-   *  <code>true</code>.
+   * Returns <code>true</code> if this geometry contains the
+   * specified geometry.
+   * <p>
+   * The <code>contains</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>Every point of the other geometry is a point of this geometry,
+   * and the interiors of the two geometries have at least one point in common.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is <code>T*****FF*</code>
+   * <li><code>g.within(this)</code>
+   * (<code>contains</code> is the inverse of <code>within</code>)
+   * </ul>
+   * An implication of the definition is that "Polygons do not
+   * contain their boundary".  In other words, if a geometry G is a subset of
+   * the points in the boundary of a polygon P, <code>P.contains(G) = false</code>
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
-   *@return        <code>true</code> if this <code>Geometry</code> contains
-   *      <code>other</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@return        <code>true</code> if this <code>Geometry</code> contains <code>g</code>
+   *
+   * @see Geometry#within
    */
   public boolean contains(Geometry g) {
     // short-circuit test
     if (! getEnvelopeInternal().contains(g.getEnvelopeInternal()))
       return false;
-    // optimizations for rectangle arguments
+    // optimization for rectangle arguments
     if (isRectangle()) {
       return RectangleContains.contains((Polygon) this, g);
     }
@@ -679,18 +760,23 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is
-   *  <UL>
-   *    <LI> T*T***T** (for two points or two surfaces)
-   *    <LI> 1*T***T** (for two curves)
-   *  </UL>
-   *  .
+   * Returns <code>true</code> if this geometry overlaps the
+   * specified geometry.
+   * <p>
+   * The <code>overlaps</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The geometries have some but not all points in common,
+   * they have the same dimension,
+   * and the intersection of the interiors of the two geometries has
+   * the same dimension as the geometries themselves.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *   <code>T*T***T**</code> (for two points or two surfaces)
+   *   or <code>1*T***T**</code> (for two curves)
+   * </ul>
+   * If the geometries are of different dimension this predicate returns <code>false</code>.
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s overlap.
-   *      For this function to return <code>true</code>, the <code>Geometry</code>
-   *      s must be two points, two curves or two surfaces.
    */
   public boolean overlaps(Geometry g) {
     // short-circuit test
@@ -700,19 +786,87 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the elements in the DE-9IM intersection
-   *  matrix for the two <code>Geometry</code>s match the elements in <code>intersectionPattern</code>
-   *  , which may be:
+   * Returns <code>true</code> if this geometry covers the
+   * specified geometry.
+   * <p>
+   * The <code>covers</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>Every point of the other geometry is a point of this geometry.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *    <code>T*****FF*</code>
+   * or <code>*T****FF*</code>
+   * or <code>***T**FF*</code>
+   * or <code>****T*FF*</code>
+   * <li><code>g.coveredBy(this)</code>
+   * (<code>covers</code> is the inverse of <code>coverdBy</code>)
+   * </ul>
+   * Note the difference between <code>covers</code> and <code>contains</code>
+   * - <code>covers</code> is a more inclusive relation.
+   * In particular, unlike <code>contains</code> it does not distinguish between
+   * points in the boundary and in the interior of geometries.
+   * For most situations, <code>covers</code> should be used in preference to <code>contains</code>.
+   * As an added benefit, <code>covers</code> is more amenable to optimization,
+   * and hence should be more performant.
+   *
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@return        <code>true</code> if this <code>Geometry</code> covers <code>g</code>
+   *
+   * @see Geometry#contains
+   * @see Geometry#coveredBy
+   */
+  public boolean covers(Geometry g) {
+    // short-circuit test
+    if (! getEnvelopeInternal().contains(g.getEnvelopeInternal()))
+      return false;
+    // optimization for rectangle arguments
+    if (isRectangle()) {
+      return getEnvelopeInternal().contains(g.getEnvelopeInternal());
+    }
+    return relate(g).isCovers();
+  }
+
+  /**
+   * Returns <code>true</code> if this geometry is covered by the
+   * specified geometry.
+   * <p>
+   * The <code>coveredBy</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>Every point of this geometry is a point of the other geometry.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *    <code>T*F**F***</code>
+   * or <code>*TF**F***</code>
+   * or <code>**FT*F***</code>
+   * or <code>**F*TF***</code>
+   * <li><code>g.covers(this)</code>
+   * (<code>coveredBy</code> is the inverse of <code>covers</code>)
+   * </ul>
+   * Note the difference between <code>coveredBy</code> and <code>within</code>
+   * - <code>coveredBy</code> is a more inclusive relation
+   *
+   *@param  g  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@return        <code>true</code> if this <code>Geometry</code> is covered by <code>g</code>
+   *
+   * @see Geometry#within
+   * @see Geometry#covers
+   */
+  public boolean coveredBy(Geometry g) {
+    return g.covers(this);
+  }
+
+  /**
+   *  Returns <code>true</code> if the elements in the DE-9IM
+   * {@link IntersectionMatrix} for the two <code>Geometry</code>s match the elements in <code>intersectionPattern</code>.
+   * The pattern is a 9-character string, with symbols drawn from the following set:
    *  <UL>
-   *    <LI> 0
-   *    <LI> 1
-   *    <LI> 2
-   *    <LI> T ( = 0, 1 or 2)
-   *    <LI> F ( = -1)
-   *    <LI> * ( = -1, 0, 1 or 2)
+   *    <LI> 0 (dimension 0)
+   *    <LI> 1 (dimension 1)
+   *    <LI> 2 (dimension 2)
+   *    <LI> T ( matches 0, 1 or 2)
+   *    <LI> F ( matches FALSE)
+   *    <LI> * ( matches any value)
    *  </UL>
-   *  For more information on the DE-9IM, see the OpenGIS Simple Features
-   *  Specification.
+   *  For more information on the DE-9IM, see the <i>OpenGIS Simple Features
+   *  Specification</i>.
    *
    *@param  other                the <code>Geometry</code> with which to compare
    *      this <code>Geometry</code>
@@ -720,16 +874,17 @@ public abstract class Geometry
    *      intersection matrix for the two <code>Geometry</code>s
    *@return                      <code>true</code> if the DE-9IM intersection
    *      matrix for the two <code>Geometry</code>s match <code>intersectionPattern</code>
+   * @see IntersectionMatrix
    */
   public boolean relate(Geometry g, String intersectionPattern) {
     return relate(g).matches(intersectionPattern);
   }
 
   /**
-   *  Returns the DE-9IM intersection matrix for the two <code>Geometry</code>s.
+   *  Returns the DE-9IM {@link IntersectionMatrix} for the two <code>Geometry</code>s.
    *
    *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
-   *@return        a matrix describing the intersections of the interiors,
+   *@return        an {@link IntersectionMatrix} describing the intersections of the interiors,
    *      boundaries and exteriors of the two <code>Geometry</code>s
    */
   public IntersectionMatrix relate(Geometry g) {
@@ -739,8 +894,15 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns <code>true</code> if the DE-9IM intersection matrix for the two
-   *  <code>Geometry</code>s is T*F**FFF*.
+   * Returns <code>true</code> if this geometry is equal to the
+   * specified geometry.
+   * <p>
+   * The <code>equals</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The two geometries have at least one point in common,
+   * and no point of either geometry lies in the exterior of the other geometry.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is T*F**FFF*
+   * </ul>
    *
    *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
    *@return        <code>true</code> if the two <code>Geometry</code>s are equal
@@ -771,42 +933,78 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns a buffer region around this <code>Geometry</code> having the given
-   *  width.
-   * The buffer of a Geometry is
-   * the Minkowski sum or difference
-   * of the Geometry with
-   * a disc of radius <code>distance</code>.
+   * Computes a buffer area around this geometry having the given
+   * width.
+   * The buffer of a Geometry is the Minkowski sum or difference
+   * of the geometry with a disc of radius <code>abs(distance)</code>.
+   * The buffer is constructed using 8 segments per quadrant to represent curves.
+   * The end cap style is <tt>CAP_ROUND</tt>.
    *
-   *@param  distance  the width of the buffer, interpreted according to the
-   *      <code>PrecisionModel</code> of the <code>Geometry</code>
-   *@return           all points whose distance from this <code>Geometry</code>
-   *      are less than or equal to <code>distance</code>
+   *@param  distance  the width of the buffer (may be positive, negative or 0)
+   *@return an area geometry representing the buffer region
+   *
+   * @see #buffer(double, int)
+   * @see #buffer(double, int, int)
    */
   public Geometry buffer(double distance) {
     return BufferOp.bufferOp(this, distance);
   }
 
   /**
-   *  Returns a buffer region around this {@link Geometry} having the given
-   *  width and with a specified number of segments used to approximate curves.
-   * The buffer of a Geometry is the Minkowski sum of the Geometry with
-   * a disc of radius <code>distance</code>.  Curves in the buffer polygon are
-   * approximated with line segments.  This method allows specifying the
-   * accuracy of that approximation.
+   * Computes a buffer area around this geometry having the given
+   * width and with a specified accuracy of approximation for circular arcs.
+   * <p>
+   * Buffer area boundaries can contain circular arcs.
+   * To represent these arcs using linear geometry they must be approximated with line segments.
+   * The <code>quadrantSegments</code> argument allows controlling the
+   * accuracy of the approximation
+   * by specifying the number of line segments used to represent a quadrant of a circle
    *
-   *@param  distance  the width of the buffer, interpreted according to the
-   *      <code>PrecisionModel</code> of the <code>Geometry</code>
-   *@param quadrantSegments the number of segments to use to approximate a quadrant of a circle
-   *@return           all points whose distance from this <code>Geometry</code>
-   *      are less than or equal to <code>distance</code>
+   *@param  distance  the width of the buffer (may be positive, negative or 0)
+   *@param quadrantSegments the number of line segments used to represent a quadrant of a circle
+   *@return an area geometry representing the buffer region
+   *
+   * @see #buffer(double)
+   * @see #buffer(double, int, int)
    */
   public Geometry buffer(double distance, int quadrantSegments) {
     return BufferOp.bufferOp(this, distance, quadrantSegments);
   }
 
   /**
-   *  Returns the smallest convex <code>Polygon</code> that contains all the
+   * Computes a buffer area around this geometry having the given
+   * width and with a specified accuracy of approximation for circular arcs,
+   * and using a specified end cap style.
+   * <p>
+   * Buffer area boundaries can contain circular arcs.
+   * To represent these arcs using linear geometry they must be approximated with line segments.
+   * The <code>quadrantSegments</code> argument allows controlling the
+   * accuracy of the approximation
+   * by specifying the number of line segments used to represent a quadrant of a circle
+   * <p>
+   * The end cap style specifies the buffer geometry that will be
+   * created at the ends of linestrings.  The styles provided are:
+   * <ul>
+   * <li><tt>BufferOp.CAP_ROUND</tt> - (default) a semi-circle
+   * <li><tt>BufferOp.CAP_BUTT</tt> - a straight line perpendicular to the end segment
+   * <li><tt>BufferOp.CAP_SQUARE</tt> - a half-square
+   * </ul>
+   *
+   *@param  distance  the width of the buffer (may be positive, negative or 0)
+   *@param quadrantSegments the number of line segments used to represent a quadrant of a circle
+   *@param endCapStyle the end cap style to use
+   *@return an area geometry representing the buffer region
+   *
+   * @see #buffer(double)
+   * @see #buffer(double, int)
+   * @see BufferOp
+   */
+  public Geometry buffer(double distance, int quadrantSegments, int endCapStyle) {
+    return BufferOp.bufferOp(this, distance, quadrantSegments, endCapStyle);
+  }
+
+  /**
+   *  Computes the smallest convex <code>Polygon</code> that contains all the
    *  points in the <code>Geometry</code>. This obviously applies only to <code>Geometry</code>
    *  s which contain 3 or more points; the results for degenerate cases are
    *  specified as follows:
@@ -840,12 +1038,13 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns a <code>Geometry</code> representing the points shared by this
+   *  Computes a <code>Geometry</code> representing the points shared by this
    *  <code>Geometry</code> and <code>other</code>.
    *
-   *@param  other  the <code>Geometry</code> with which to compute the
+   * @param  other  the <code>Geometry</code> with which to compute the
    *      intersection
-   *@return        the points common to the two <code>Geometry</code>s
+   * @return        the points common to the two <code>Geometry</code>s
+   * @throws TopologyException if a robustness error occurs
    */
   public Geometry intersection(Geometry other) {
     checkNotGeometryCollection(this);
@@ -854,12 +1053,13 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns a <code>Geometry</code> representing all the points in this <code>Geometry</code>
+   *  Computes a <code>Geometry</code> representing all the points in this <code>Geometry</code>
    *  and <code>other</code>.
    *
    *@param  other  the <code>Geometry</code> with which to compute the union
    *@return        a set combining the points of this <code>Geometry</code> and
    *      the points of <code>other</code>
+   * @throws TopologyException if a robustness error occurs
    */
   public Geometry union(Geometry other) {
     checkNotGeometryCollection(this);
@@ -868,7 +1068,7 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns a <code>Geometry</code> representing the points making up this
+   *  Computes a <code>Geometry</code> representing the points making up this
    *  <code>Geometry</code> that do not make up <code>other</code>. This method
    *  returns the closure of the resultant <code>Geometry</code>.
    *
@@ -876,6 +1076,7 @@ public abstract class Geometry
    *      difference
    *@return        the point set difference of this <code>Geometry</code> with
    *      <code>other</code>
+   * @throws TopologyException if a robustness error occurs
    */
   public Geometry difference(Geometry other) {
     checkNotGeometryCollection(this);
@@ -893,6 +1094,7 @@ public abstract class Geometry
    *      difference
    *@return        the point set symmetric difference of this <code>Geometry</code>
    *      with <code>other</code>
+   * @throws TopologyException if a robustness error occurs
    */
   public Geometry symDifference(Geometry other) {
     checkNotGeometryCollection(this);
@@ -901,24 +1103,24 @@ public abstract class Geometry
   }
 
   /**
-   *  Returns true if the two <code>Geometry</code>s are exactly equal,
-   * up to a specified tolerance.
-   * Two Geometries are exactly within a tolerance equal iff:
+   * Returns true if the two <code>Geometry</code>s are exactly equal,
+   * up to a specified distance tolerance.
+   * Two Geometries are exactly equal within a distance tolerance
+   * if and only if:
    * <ul>
    * <li>they have the same class
-   * <li>they have the same values of Coordinates,
-   * within the given tolerance distance, in their internal
-   * Coordinate lists, in exactly the same order.
+   * <li>they have the same values for their vertices,
+   * within the given tolerance distance, in exactly the same order.
    * </ul>
    * If this and the other <code>Geometry</code>s are
-   *  composites and any children are not <code>Geometry</code>s, returns
-   *  false.
+   * composites and any children are not <code>Geometry</code>s, returns
+   * <code>false</code>.
    *
-   *@param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
-   *@parm tolerance distance at or below which two Coordinates will be considered
-   * equal
-   *@return        <code>true</code> if this and the other <code>Geometry</code>
-   *      are of the same class and have equal internal data.
+   * @param  other  the <code>Geometry</code> with which to compare this <code>Geometry</code>
+   * @parm tolerance distance at or below which two <code>Coordinate</code>s
+   *   are considered equal
+   * @return <code>true</code> if this and the other <code>Geometry</code>
+   *   are of the same class and have equal internal data.
    */
   public abstract boolean equalsExact(Geometry other, double tolerance);
 
@@ -1046,6 +1248,53 @@ public abstract class Geometry
     return compareToSameClass(o);
   }
 
+  /**
+   *  Returns whether this <code>Geometry</code> is greater than, equal to,
+   *  or less than another <code>Geometry</code>,
+   * using the given {@link CoordinateSequenceComparator}.
+   * <P>
+   *
+   *  If their classes are different, they are compared using the following
+   *  ordering:
+   *  <UL>
+   *    <LI> Point (lowest)
+   *    <LI> MultiPoint
+   *    <LI> LineString
+   *    <LI> LinearRing
+   *    <LI> MultiLineString
+   *    <LI> Polygon
+   *    <LI> MultiPolygon
+   *    <LI> GeometryCollection (highest)
+   *  </UL>
+   *  If the two <code>Geometry</code>s have the same class, their first
+   *  elements are compared. If those are the same, the second elements are
+   *  compared, etc.
+   *
+   *@param  o  a <code>Geometry</code> with which to compare this <code>Geometry</code>
+   *@param comp a <code>CoordinateSequenceComparator</code>
+   *
+   *@return    a positive number, 0, or a negative number, depending on whether
+   *      this object is greater than, equal to, or less than <code>o</code>, as
+   *      defined in "Normal Form For Geometry" in the JTS Technical
+   *      Specifications
+   */
+  public int compareTo(Object o, CoordinateSequenceComparator comp) {
+    Geometry other = (Geometry) o;
+    if (getClassSortIndex() != other.getClassSortIndex()) {
+      return getClassSortIndex() - other.getClassSortIndex();
+    }
+    if (isEmpty() && other.isEmpty()) {
+      return 0;
+    }
+    if (isEmpty()) {
+      return -1;
+    }
+    if (other.isEmpty()) {
+      return 1;
+    }
+    return compareToSameClass(o, comp);
+  }
+
   /**
    *  Returns whether the two <code>Geometry</code>s are equal, from the point
    *  of view of the <code>equalsExact</code> method. Called by <code>equalsExact</code>
@@ -1102,6 +1351,20 @@ public abstract class Geometry
    */
   protected abstract int compareToSameClass(Object o);
 
+  /**
+   *  Returns whether this <code>Geometry</code> is greater than, equal to,
+   *  or less than another <code>Geometry</code> of the same class.
+   * using the given {@link CoordinateSequenceComparator}.
+   *
+   *@param  o  a <code>Geometry</code> having the same class as this <code>Geometry</code>
+   *@param comp a <code>CoordinateSequenceComparator</code>
+   *@return    a positive number, 0, or a negative number, depending on whether
+   *      this object is greater than, equal to, or less than <code>o</code>, as
+   *      defined in "Normal Form For Geometry" in the JTS Technical
+   *      Specifications
+   */
+  protected abstract int compareToSameClass(Object o, CoordinateSequenceComparator comp);
+
   /**
    *  Returns the first non-zero result of <code>compareTo</code> encountered as
    *  the two <code>Collection</code>s are iterated over. If, by the time one of
