@@ -44,7 +44,7 @@ import com.vividsolutions.jts.util.Assert;
  * Supplies a set of utility methods for building Geometry objects from lists
  * of Coordinates.
  *
- * @version 1.6
+ * @version 1.7
  */
 public class GeometryFactory
     implements Serializable
@@ -337,7 +337,9 @@ public class GeometryFactory
    */
 
   public MultiPoint createMultiPoint(Coordinate[] coordinates) {
-      return createMultiPoint(coordinates != null ? getCoordinateSequenceFactory().create(coordinates) : null);
+      return createMultiPoint(coordinates != null
+                              ? getCoordinateSequenceFactory().create(coordinates)
+                              : null);
   }
 
   /**
@@ -347,13 +349,13 @@ public class GeometryFactory
    */
   public MultiPoint createMultiPoint(CoordinateSequence coordinates) {
     if (coordinates == null) {
-      coordinates = getCoordinateSequenceFactory().create(new Coordinate[]{});
+      return createMultiPoint(new Point[0]);
     }
-    ArrayList points = new ArrayList();
+    Point[] points = new Point[coordinates.size()];
     for (int i = 0; i < coordinates.size(); i++) {
-      points.add(createPoint(coordinates.getCoordinate(i)));
+      points[i] = createPoint(coordinates.getCoordinate(i));
     }
-    return createMultiPoint((Point[]) points.toArray(new Point[]{}));
+    return createMultiPoint(points);
   }
 
 
@@ -475,15 +477,15 @@ public class GeometryFactory
   }
 
 
-    public int getSRID() {
-		return SRID;
-	}
+  public int getSRID() {
+    return SRID;
+  }
 
-    private int SRID;
+  private int SRID;
 
-    public CoordinateSequenceFactory getCoordinateSequenceFactory() {
-        return coordinateSequenceFactory;
-    }
+  public CoordinateSequenceFactory getCoordinateSequenceFactory() {
+    return coordinateSequenceFactory;
+  }
 
 }
 
