@@ -52,7 +52,7 @@ package com.vividsolutions.jts.geom;
  *  HREF="http://www.opengis.org/techno/specs.htm">OpenGIS Simple Features
  *  Specification for SQL</A> .
  *
- *@version 1.6
+ *@version 1.7
  */
 public class IntersectionMatrix implements Cloneable {
   /**
@@ -332,21 +332,29 @@ public class IntersectionMatrix implements Cloneable {
   }
 
   /**
-   *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
-   *  <UL>
-   *    <LI> T*T****** (for a point and a curve, a point and an area or a line
-   *    and an area)
-   *    <LI> 0******** (for two curves)
-   *  </UL>
-   *  .
+   * Returns <code>true</code> if this geometry crosses the
+   * specified geometry.
+   * <p>
+   * The <code>crosses</code> predicate has the following equivalent definitions:
+   * <ul>
+   * <li>The geometries have some but not all interior points in common.
+   * <li>The DE-9IM Intersection Matrix for the two geometries is
+   *   <ul>
+   *    <li>T*T****** (for P/L, P/A, and L/A situations)
+   *    <li>T*****T** (for L/P, L/A, and A/L situations)
+   *    <li>0******** (for L/L situations)
+   *   </ul>
+   * </ul>
+   * For any other combination of dimensions this predicate returns <code>false</code>.
+   * <p>
+   * The SFS defined this predicate only for P/L, P/A, L/L, and L/A situations.
+   * JTS extends the definition to apply to L/P, A/P and A/L situations as well.
+   * This makes the relation symmetric.
    *
    *@param  dimensionOfGeometryA  the dimension of the first <code>Geometry</code>
    *@param  dimensionOfGeometryB  the dimension of the second <code>Geometry</code>
-   *@return                       <code>true</code> if the two <code>Geometry</code>
-   *      s related by this <code>IntersectionMatrix</code> cross. For this
-   *      function to return <code>true</code>, the <code>Geometry</code>s must
-   *      be a point and a curve; a point and a surface; two curves; or a curve
-   *      and a surface.
+   *@return                       <code>true</code> if the two <code>Geometry</code>s
+   *      related by this <code>IntersectionMatrix</code> cross.
    */
   public boolean isCrosses(int dimensionOfGeometryA, int dimensionOfGeometryB) {
     if ((dimensionOfGeometryA == Dimension.P && dimensionOfGeometryB == Dimension.L) ||
@@ -393,6 +401,50 @@ public class IntersectionMatrix implements Cloneable {
         matrix[Location.EXTERIOR][Location.BOUNDARY] == Dimension.FALSE;
   }
 
+  /**
+   *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
+   *    <code>T*****FF*</code>
+   * or <code>*T****FF*</code>
+   * or <code>***T**FF*</code>
+   * or <code>****T*FF*</code>
+   *
+   *@return    <code>true</code> if the first <code>Geometry</code> covers the
+   *      second
+   */
+  public boolean isCovers() {
+    boolean hasPointInCommon =
+        matches(matrix[Location.INTERIOR][Location.INTERIOR], 'T')
+        || matches(matrix[Location.INTERIOR][Location.BOUNDARY], 'T')
+        || matches(matrix[Location.BOUNDARY][Location.INTERIOR], 'T')
+        || matches(matrix[Location.BOUNDARY][Location.BOUNDARY], 'T');
+
+    return hasPointInCommon &&
+        matrix[Location.EXTERIOR][Location.INTERIOR] == Dimension.FALSE &&
+        matrix[Location.EXTERIOR][Location.BOUNDARY] == Dimension.FALSE;
+  }
+
+  /**
+   *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
+   *    <code>T*F**F***</code>
+   * or <code>*TF**F***</code>
+   * or <code>**FT*F***</code>
+   * or <code>**F*TF***</code>
+   *
+   *@return    <code>true</code> if the first <code>Geometry</code> covers the
+   *      second
+   */
+  public boolean isCoveredBy() {
+    boolean hasPointInCommon =
+        matches(matrix[Location.INTERIOR][Location.INTERIOR], 'T')
+        || matches(matrix[Location.INTERIOR][Location.BOUNDARY], 'T')
+        || matches(matrix[Location.BOUNDARY][Location.INTERIOR], 'T')
+        || matches(matrix[Location.BOUNDARY][Location.BOUNDARY], 'T');
+
+    return hasPointInCommon &&
+        matrix[Location.INTERIOR][Location.EXTERIOR] == Dimension.FALSE &&
+        matrix[Location.BOUNDARY][Location.EXTERIOR] == Dimension.FALSE;
+  }
+
   /**
    *  Returns <code>true</code> if this <code>IntersectionMatrix</code> is
    *  T*F**FFF*.
