@@ -1,5 +1,3 @@
-
-
 /*
  * The JTS Topology Suite is a collection of Java classes that
  * implement the fundamental operations required to validate a given
@@ -35,9 +33,9 @@
 package com.vividsolutions.jts.io;
 
 import com.vividsolutions.jts.geom.*;
+import com.vividsolutions.jts.util.*;
 import com.vividsolutions.jts.io.ParseException;
 
-import com.vividsolutions.jts.util.Assert;
 import java.io.IOException;
 import java.io.Reader;
 import java.io.StreamTokenizer;
@@ -45,50 +43,100 @@ import java.io.StringReader;
 import java.util.ArrayList;
 
 /**
- *  Converts a Well-Known Text string to a <code>Geometry</code>.
+ * Converts a geometry in Well-Known Text format to a {@link Geometry}.
  * <p>
- *  The <code>WKTReader</code> allows
- *  extracting <code>Geometry</code> objects from either input streams or
- *  internal strings. This allows it to function as a parser to read <code>Geometry</code>
+ * <code>WKTReader</code> supports
+ * extracting <code>Geometry</code> objects from either {@link Reader}s or
+ *  {@link String}s. This allows it to function as a parser to read <code>Geometry</code>
  *  objects from text blocks embedded in other data formats (e.g. XML). <P>
  * <p>
- * The Well-known
- *  Text format is defined in the <A HREF="http://www.opengis.org/techno/specs.htm">
- *  OpenGIS Simple Features Specification for SQL</A> . <P>
- * <p>
- *  <B>Note: </B> There is an inconsistency in the SFS. The WKT grammar states
- *  that <code>MultiPoints</code> are represented by <code>MULTIPOINT ( ( x y), (x y) )</code>
- *  , but the examples show <code>MultiPoint</code>s as <code>MULTIPOINT ( x y, x y )</code>
- *  . Other implementations follow the latter syntax, so JTS will adopt it as
- *  well.
- *
- *  A <code>WKTReader</code> is parameterized by a <code>GeometryFactory</code>
- *  , to allow it to create <code>Geometry</code> objects of the appropriate
- *  implementation. In particular, the <code>GeometryFactory</code> will
- *  determine the <code>PrecisionModel</code> and <code>SRID</code> that is
+ *  A <code>WKTReader</code> is parameterized by a <code>GeometryFactory</code>,
+ *  to allow it to create <code>Geometry</code> objects of the appropriate
+ *  implementation. In particular, the <code>GeometryFactory</code>
+ *  determines the <code>PrecisionModel</code> and <code>SRID</code> that is
  *  used. <P>
  *
- *  The <code>WKTReader</code> will convert the input numbers to the precise
+ *  The <code>WKTReader</code> converts all input numbers to the precise
  *  internal representation.
  *
- *  Reads non-standard "LINEARRING" tags.
+ * <h3>Notes:</h3>
+ * <ul>
+ * <li>The reader supports non-standard "LINEARRING" tags.
+ * <li>The reader uses Double.parseDouble to perform the conversion of ASCII
+ * numbers to floating point.  This means it supports the Java
+ * syntax for floating point literals (including scientific notation).
+ * </ul>
  *
- *@version 1.6
+ * <h3>Syntax</h3>
+ * The following syntax specification describes the version of Well-Known Text
+ * supported by JTS.
+ * (The specification uses a syntax language similar to that used in
+ * the C and Java language specifications.)
+ * <p>
+ *
+ * <blockquote><pre>
+ * <i>WKTGeometry:</i> one of<i>
+ *
+ *       WKTPoint  WKTLineString  WKTLinearRing  WKTPolygon
+ *       WKTMultiPoint  WKTMultiLineString  WKTMultiPolygon
+ *       WKTGeometryCollection</i>
+ *
+ * <i>WKTPoint:</i> <b>POINT ( </b><i>Coordinate</i> <b>)</b>
+ *
+ * <i>WKTLineString:</i> <b>LINESTRING</b> <i>CoordinateSequence</i>
+ *
+ * <i>WKTLinearRing:</i> <b>LINEARRING</b> <i>CoordinateSequence</i>
+ *
+ * <i>WKTPolygon:</i> <b>POLYGON</b> <i>CoordinateSequenceList</i>
+ *
+ * <i>WKTMultiPoint:</i> <b>MULTIPOINT</b> <i>CoordinateSequence</i>
+ *
+ * <i>WKTMultiLineString:</i> <b>MULTILINESTRING</b> <i>CoordinateSequenceList</i>
+ *
+ * <i>WKTMultiPolygon:</i>
+ *         <b>MULTIPOLYGON (</b> <i>CoordinateSequenceList {</i> , <i>CoordinateSequenceList }</i> <b>)</b>
+ *
+ * <i>WKTGeometryCollection: </i>
+ *         <b>GEOMETRYCOLLECTION (</b> <i>WKTGeometry {</i> , <i>WKTGeometry }</i> <b>)</b>
+ *
+ * <i>CoordinateSequenceList:</i>
+ *         <b>(</b> <i>CoordinateSequence {</i> <b>,</b> <i>CoordinateSequence }</i> <b>)</b>
+ *
+ * <i>CoordinateSequence:</i>
+ *         <b>(</b> <i>Coordinate {</i> , <i>Coordinate }</i> <b>)</b>
+ *
+ * <i>Coordinate:
+ *         Number Number Number<sub>opt</sub></i>
+ *
+ * <i>Number:</i> A Java-style floating-point number
+ *
+ * </pre></blockquote>
+ *
+ *
+ *@version 1.7
+ * @see WKTWriter
  */
-public class WKTReader {
+public class WKTReader
+{
+  private static final String EMPTY = "EMPTY";
+  private static final String COMMA = ",";
+  private static final String L_PAREN = "(";
+  private static final String R_PAREN = ")";
+
   private GeometryFactory geometryFactory;
   private PrecisionModel precisionModel;
+  private StreamTokenizer tokenizer;
 
   /**
-   * Creates a WKTReader that creates objects using a basic GeometryFactory.
+   * Creates a reader that creates objects using the default {@link GeometryFactory}.
    */
   public WKTReader() {
     this(new GeometryFactory());
   }
 
   /**
-   *  Creates a <code>WKTReader</code> that creates objects using the given
-   *  <code>GeometryFactory</code>.
+   *  Creates a reader that creates objects using the given
+   *  {@link GeometryFactory}.
    *
    *@param  geometryFactory  the factory used to create <code>Geometry</code>s.
    */
@@ -97,18 +145,17 @@ public class WKTReader {
     precisionModel = geometryFactory.getPrecisionModel();
   }
 
-  
-
-	/**
-     * Converts a Well-known Text representation to a <code>Geometry</code>.
-     * 
-     * @param wellKnownText
-     *            one or more <Geometry Tagged Text>strings (see the OpenGIS
-     *            Simple Features Specification) separated by whitespace
-     * @return a <code>Geometry</code> specified by <code>wellKnownText</code>
-     * @throws ParseException
-     *             if a parsing problem occurs
-	 */
+  /**
+   * Reads a Well-Known Text representation of a {@link Geometry}
+   * from a {@link String}.
+   *
+   * @param wellKnownText
+   *            one or more <Geometry Tagged Text>strings (see the OpenGIS
+   *            Simple Features Specification) separated by whitespace
+   * @return a <code>Geometry</code> specified by <code>wellKnownText</code>
+   * @throws ParseException
+   *             if a parsing problem occurs
+   */
   public Geometry read(String wellKnownText) throws ParseException {
     StringReader reader = new StringReader(wellKnownText);
     try {
@@ -120,7 +167,8 @@ public class WKTReader {
   }
 
   /**
-   *  Converts a Well-known Text representation to a <code>Geometry</code>.
+   * Reads a Well-Known Text representation of a {@link Geometry}
+   * from a {@link Reader}.
    *
    *@param  reader           a Reader which will return a <Geometry Tagged Text>
    *      string (see the OpenGIS Simple Features Specification)
@@ -128,9 +176,21 @@ public class WKTReader {
    *@throws  ParseException  if a parsing problem occurs
    */
   public Geometry read(Reader reader) throws ParseException {
-    StreamTokenizer tokenizer = new StreamTokenizer(reader);
+    tokenizer = new StreamTokenizer(reader);
+    // set tokenizer to NOT parse numbers
+    tokenizer.resetSyntax();
+    tokenizer.wordChars('a', 'z');
+    tokenizer.wordChars('A', 'Z');
+    tokenizer.wordChars(128 + 32, 255);
+    tokenizer.wordChars('0', '9');
+    tokenizer.wordChars('-', '-');
+    tokenizer.wordChars('+', '+');
+    tokenizer.wordChars('.', '.');
+    tokenizer.whitespaceChars(0, ' ');
+    tokenizer.commentChar('#');
+
     try {
-      return readGeometryTaggedText(tokenizer);
+      return readGeometryTaggedText();
     }
     catch (IOException e) {
       throw new ParseException(e.toString());
@@ -138,145 +198,138 @@ public class WKTReader {
   }
 
   /**
-   *  Returns the next array of <code>Coordinate</code>s in the stream.
+   * Returns the next array of <code>Coordinate</code>s in the stream.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
-   *      format. The next element returned by the stream should be "(" (the
-   *      beginning of "(x1 y1, x2 y2, ..., xn yn)") or "EMPTY".
+   *      format. The next element returned by the stream should be L_PAREN (the
+   *      beginning of "(x1 y1, x2 y2, ..., xn yn)") or EMPTY.
    *@return                  the next array of <code>Coordinate</code>s in the
-   *      stream, or an empty array if "EMPTY" is the next element returned by
+   *      stream, or an empty array if EMPTY is the next element returned by
    *      the stream.
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private Coordinate[] getCoordinates(StreamTokenizer tokenizer)
+  private Coordinate[] getCoordinates()
       throws IOException, ParseException
   {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
       return new Coordinate[]{};
     }
     ArrayList coordinates = new ArrayList();
-    coordinates.add(getPreciseCoordinate(tokenizer));
-    nextToken = getNextCloserOrComma(tokenizer);
-    while (nextToken.equals(",")) {
-      coordinates.add(getPreciseCoordinate(tokenizer));
-      nextToken = getNextCloserOrComma(tokenizer);
+    coordinates.add(getPreciseCoordinate());
+    nextToken = getNextCloserOrComma();
+    while (nextToken.equals(COMMA)) {
+      coordinates.add(getPreciseCoordinate());
+      nextToken = getNextCloserOrComma();
     }
     Coordinate[] array = new Coordinate[coordinates.size()];
     return (Coordinate[]) coordinates.toArray(array);
   }
 
-  private Coordinate getPreciseCoordinate(StreamTokenizer tokenizer)
+  private Coordinate getPreciseCoordinate()
       throws IOException, ParseException
   {
     Coordinate coord = new Coordinate();
-    coord.x = getNextNumber(tokenizer);
-    coord.y = getNextNumber(tokenizer);
-    if (isNumberNext(tokenizer)) {
-        coord.z = getNextNumber(tokenizer);
+    coord.x = getNextNumber();
+    coord.y = getNextNumber();
+    if (isNumberNext()) {
+        coord.z = getNextNumber();
     }
     precisionModel.makePrecise(coord);
     return coord;
   }
-  private boolean isNumberNext(StreamTokenizer tokenizer) throws IOException {
-      try {
-          return tokenizer.nextToken() == StreamTokenizer.TT_NUMBER;
-      }
-      finally {
-          tokenizer.pushBack();
-      }
+
+  private boolean isNumberNext() throws IOException {
+    int type = tokenizer.nextToken();
+    tokenizer.pushBack();
+    return type == StreamTokenizer.TT_WORD;
   }
+
   /**
-   *  Returns the next number in the stream.
+   * Parses the next number in the stream.
+   * Numbers with exponents are handled.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
    *      format. The next token must be a number.
    *@return                  the next number in the stream
-   *@throws  ParseException  if the next token is not a number
+   *@throws  ParseException  if the next token is not a valid number
    *@throws  IOException     if an I/O error occurs
    */
-  private double getNextNumber(StreamTokenizer tokenizer) throws IOException,
+  private double getNextNumber() throws IOException,
       ParseException {
     int type = tokenizer.nextToken();
     switch (type) {
-      case StreamTokenizer.TT_EOF:
-        throw new ParseException("Expected number but encountered end of stream");
-      case StreamTokenizer.TT_EOL:
-        throw new ParseException("Expected number but encountered end of line");
-      case StreamTokenizer.TT_NUMBER:
-        return tokenizer.nval;
       case StreamTokenizer.TT_WORD:
-        throw new ParseException("Expected number but encountered word: " +
-            tokenizer.sval);
-      case '(':
-        throw new ParseException("Expected number but encountered '('");
-      case ')':
-        throw new ParseException("Expected number but encountered ')'");
-      case ',':
-        throw new ParseException("Expected number but encountered ','");
+      {
+        try {
+          return Double.parseDouble(tokenizer.sval);
+        }
+        catch (NumberFormatException ex) {
+          throw new ParseException("Invalid number: " + tokenizer.sval);
+        }
+      }
     }
-    Assert.shouldNeverReachHere("Encountered unexpected StreamTokenizer type: "
-         + type);
-    return 0;
+    parseError("number");
+    return 0.0;
   }
-
   /**
-   *  Returns the next "EMPTY" or "(" in the stream as uppercase text.
+   *  Returns the next EMPTY or L_PAREN in the stream as uppercase text.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
-   *      format. The next token must be "EMPTY" or "(".
-   *@return                  the next "EMPTY" or "(" in the stream as uppercase
+   *      format. The next token must be EMPTY or L_PAREN.
+   *@return                  the next EMPTY or L_PAREN in the stream as uppercase
    *      text.
-   *@throws  ParseException  if the next token is not "EMPTY" or "("
+   *@throws  ParseException  if the next token is not EMPTY or L_PAREN
    *@throws  IOException     if an I/O error occurs
    */
-  private String getNextEmptyOrOpener(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextWord = getNextWord(tokenizer);
-    if (nextWord.equals("EMPTY") || nextWord.equals("(")) {
+  private String getNextEmptyOrOpener() throws IOException, ParseException {
+    String nextWord = getNextWord();
+    if (nextWord.equals(EMPTY) || nextWord.equals(L_PAREN)) {
       return nextWord;
     }
-    throw new ParseException("Expected 'EMPTY' or '(' but encountered '" +
-        nextWord + "'");
+    parseError(EMPTY + " or " + L_PAREN);
+    return null;
   }
 
   /**
-   *  Returns the next ")" or "," in the stream.
+   *  Returns the next R_PAREN or COMMA in the stream.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
-   *      format. The next token must be ")" or ",".
-   *@return                  the next ")" or "," in the stream
-   *@throws  ParseException  if the next token is not ")" or ","
+   *      format. The next token must be R_PAREN or COMMA.
+   *@return                  the next R_PAREN or COMMA in the stream
+   *@throws  ParseException  if the next token is not R_PAREN or COMMA
    *@throws  IOException     if an I/O error occurs
    */
-  private String getNextCloserOrComma(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextWord = getNextWord(tokenizer);
-    if (nextWord.equals(",") || nextWord.equals(")")) {
+  private String getNextCloserOrComma() throws IOException, ParseException {
+    String nextWord = getNextWord();
+    if (nextWord.equals(COMMA) || nextWord.equals(R_PAREN)) {
       return nextWord;
     }
-    throw new ParseException("Expected ')' or ',' but encountered '" + nextWord
-         + "'");
+    parseError(COMMA + " or " + R_PAREN);
+    return null;
   }
 
   /**
-   *  Returns the next ")" in the stream.
+   *  Returns the next R_PAREN in the stream.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
-   *      format. The next token must be ")".
-   *@return                  the next ")" in the stream
-   *@throws  ParseException  if the next token is not ")"
+   *      format. The next token must be R_PAREN.
+   *@return                  the next R_PAREN in the stream
+   *@throws  ParseException  if the next token is not R_PAREN
    *@throws  IOException     if an I/O error occurs
    */
-  private String getNextCloser(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextWord = getNextWord(tokenizer);
-    if (nextWord.equals(")")) {
+  private String getNextCloser() throws IOException, ParseException {
+    String nextWord = getNextWord();
+    if (nextWord.equals(R_PAREN)) {
       return nextWord;
     }
-    throw new ParseException("Expected ')' but encountered '" + nextWord + "'");
+    parseError(R_PAREN);
+    return null;
   }
 
   /**
-   *  Returns the next word in the stream as uppercase text.
+   *  Returns the next word in the stream.
    *
    *@param  tokenizer        tokenizer over a stream of text in Well-known Text
    *      format. The next token must be a word.
@@ -284,29 +337,62 @@ public class WKTReader {
    *@throws  ParseException  if the next token is not a word
    *@throws  IOException     if an I/O error occurs
    */
-  private String getNextWord(StreamTokenizer tokenizer) throws IOException, ParseException {
+  private String getNextWord() throws IOException, ParseException {
     int type = tokenizer.nextToken();
     switch (type) {
-      case StreamTokenizer.TT_EOF:
-        throw new ParseException("Expected word but encountered end of stream");
-      case StreamTokenizer.TT_EOL:
-        throw new ParseException("Expected word but encountered end of line");
-      case StreamTokenizer.TT_NUMBER:
-        throw new ParseException("Expected word but encountered number: " +
-            tokenizer.nval);
-      case StreamTokenizer.TT_WORD:
-        return tokenizer.sval.toUpperCase();
-      case '(':
-        return "(";
-      case ')':
-        return ")";
-      case ',':
-        return ",";
+    case StreamTokenizer.TT_WORD:
+
+      String word = tokenizer.sval;
+      if (word.equalsIgnoreCase(EMPTY))
+          return EMPTY;
+      return word;
+
+    case '(': return L_PAREN;
+    case ')': return R_PAREN;
+    case ',': return COMMA;
     }
-    Assert.shouldNeverReachHere("Encountered unexpected StreamTokenizer type: " + type);
+    parseError("word");
     return null;
   }
 
+  /**
+   * Throws a formatted ParseException for the current token.
+   *
+   * @param expected a description of what was expected
+   * @throws ParseException
+   * @throws AssertionFailedException if an invalid token is encountered
+   */
+  private void parseError(String expected)
+      throws ParseException
+  {
+    // throws Asserts for tokens that should never be seen
+    if (tokenizer.ttype == StreamTokenizer.TT_NUMBER)
+      Assert.shouldNeverReachHere("Unexpected NUMBER token");
+    if (tokenizer.ttype == StreamTokenizer.TT_EOL)
+      Assert.shouldNeverReachHere("Unexpected EOL token");
+
+    String tokenStr = tokenString();
+    throw new ParseException("Expected " + expected + " but found " + tokenStr);
+  }
+
+  /**
+   * Gets a description of the current token
+   *
+   * @return a description of the current token
+   */
+  private String tokenString()
+  {
+    switch (tokenizer.ttype) {
+      case StreamTokenizer.TT_NUMBER:
+        return "<NUMBER>";
+      case StreamTokenizer.TT_EOL:
+        return "End-of-Line";
+      case StreamTokenizer.TT_EOF: return "End-of-Stream";
+      case StreamTokenizer.TT_WORD: return "'" + tokenizer.sval + "'";
+    }
+    return "'" + (char) tokenizer.ttype + "'";
+  }
+
   /**
    *  Creates a <code>Geometry</code> using the next token in the stream.
    *
@@ -319,33 +405,33 @@ public class WKTReader {
    *      token was encountered
    *@throws  IOException     if an I/O error occurs
    */
-  private Geometry readGeometryTaggedText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String type = getNextWord(tokenizer);
+  private Geometry readGeometryTaggedText() throws IOException, ParseException {
+    String type = getNextWord();
     if (type.equals("POINT")) {
-      return readPointText(tokenizer);
+      return readPointText();
     }
-    else if (type.equals("LINESTRING")) {
-      return readLineStringText(tokenizer);
+    else if (type.equalsIgnoreCase("LINESTRING")) {
+      return readLineStringText();
     }
-    else if (type.equals("LINEARRING")) {
-      return readLinearRingText(tokenizer);
+    else if (type.equalsIgnoreCase("LINEARRING")) {
+      return readLinearRingText();
     }
-    else if (type.equals("POLYGON")) {
-      return readPolygonText(tokenizer);
+    else if (type.equalsIgnoreCase("POLYGON")) {
+      return readPolygonText();
     }
-    else if (type.equals("MULTIPOINT")) {
-      return readMultiPointText(tokenizer);
+    else if (type.equalsIgnoreCase("MULTIPOINT")) {
+      return readMultiPointText();
     }
-    else if (type.equals("MULTILINESTRING")) {
-      return readMultiLineStringText(tokenizer);
+    else if (type.equalsIgnoreCase("MULTILINESTRING")) {
+      return readMultiLineStringText();
     }
-    else if (type.equals("MULTIPOLYGON")) {
-      return readMultiPolygonText(tokenizer);
+    else if (type.equalsIgnoreCase("MULTIPOLYGON")) {
+      return readMultiPolygonText();
     }
-    else if (type.equals("GEOMETRYCOLLECTION")) {
-      return readGeometryCollectionText(tokenizer);
+    else if (type.equalsIgnoreCase("GEOMETRYCOLLECTION")) {
+      return readGeometryCollectionText();
     }
-    throw new ParseException("Unknown type: " + type);
+    throw new ParseException("Unknown geometry type: " + type);
   }
 
   /**
@@ -358,13 +444,13 @@ public class WKTReader {
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private Point readPointText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+  private Point readPointText() throws IOException, ParseException {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
       return geometryFactory.createPoint((Coordinate)null);
     }
-    Point point = geometryFactory.createPoint(getPreciseCoordinate(tokenizer));
-    getNextCloser(tokenizer);
+    Point point = geometryFactory.createPoint(getPreciseCoordinate());
+    getNextCloser();
     return point;
   }
 
@@ -378,8 +464,8 @@ public class WKTReader {
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private LineString readLineStringText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    return geometryFactory.createLineString(getCoordinates(tokenizer));
+  private LineString readLineStringText() throws IOException, ParseException {
+    return geometryFactory.createLineString(getCoordinates());
   }
 
   /**
@@ -394,10 +480,10 @@ public class WKTReader {
    *      do not form a closed linestring, or if an unexpected token was
    *      encountered
    */
-  private LinearRing readLinearRingText(StreamTokenizer tokenizer)
+  private LinearRing readLinearRingText()
     throws IOException, ParseException
   {
-    return geometryFactory.createLinearRing(getCoordinates(tokenizer));
+    return geometryFactory.createLinearRing(getCoordinates());
   }
 
   /**
@@ -410,8 +496,8 @@ public class WKTReader {
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private MultiPoint readMultiPointText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    return geometryFactory.createMultiPoint(toPoints(getCoordinates(tokenizer)));
+  private MultiPoint readMultiPointText() throws IOException, ParseException {
+    return geometryFactory.createMultiPoint(toPoints(getCoordinates()));
   }
 
   /**
@@ -443,19 +529,19 @@ public class WKTReader {
    *      token was encountered.
    *@throws  IOException     if an I/O error occurs
    */
-  private Polygon readPolygonText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+  private Polygon readPolygonText() throws IOException, ParseException {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
         return geometryFactory.createPolygon(geometryFactory.createLinearRing(
             new Coordinate[]{}), new LinearRing[]{});
     }
     ArrayList holes = new ArrayList();
-    LinearRing shell = readLinearRingText(tokenizer);
-    nextToken = getNextCloserOrComma(tokenizer);
-    while (nextToken.equals(",")) {
-      LinearRing hole = readLinearRingText(tokenizer);
+    LinearRing shell = readLinearRingText();
+    nextToken = getNextCloserOrComma();
+    while (nextToken.equals(COMMA)) {
+      LinearRing hole = readLinearRingText();
       holes.add(hole);
-      nextToken = getNextCloserOrComma(tokenizer);
+      nextToken = getNextCloserOrComma();
     }
     LinearRing[] array = new LinearRing[holes.size()];
     return geometryFactory.createPolygon(shell, (LinearRing[]) holes.toArray(array));
@@ -471,19 +557,19 @@ public class WKTReader {
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private com.vividsolutions.jts.geom.MultiLineString readMultiLineStringText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+  private com.vividsolutions.jts.geom.MultiLineString readMultiLineStringText() throws IOException, ParseException {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
       return geometryFactory.createMultiLineString(new LineString[]{});
     }
     ArrayList lineStrings = new ArrayList();
-    LineString lineString = readLineStringText(tokenizer);
+    LineString lineString = readLineStringText();
     lineStrings.add(lineString);
-    nextToken = getNextCloserOrComma(tokenizer);
-    while (nextToken.equals(",")) {
-      lineString = readLineStringText(tokenizer);
+    nextToken = getNextCloserOrComma();
+    while (nextToken.equals(COMMA)) {
+      lineString = readLineStringText();
       lineStrings.add(lineString);
-      nextToken = getNextCloserOrComma(tokenizer);
+      nextToken = getNextCloserOrComma();
     }
     LineString[] array = new LineString[lineStrings.size()];
     return geometryFactory.createMultiLineString((LineString[]) lineStrings.toArray(array));
@@ -500,19 +586,19 @@ public class WKTReader {
    *@throws  IOException     if an I/O error occurs
    *@throws  ParseException  if an unexpected token was encountered
    */
-  private MultiPolygon readMultiPolygonText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+  private MultiPolygon readMultiPolygonText() throws IOException, ParseException {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
       return geometryFactory.createMultiPolygon(new Polygon[]{});
     }
     ArrayList polygons = new ArrayList();
-    Polygon polygon = readPolygonText(tokenizer);
+    Polygon polygon = readPolygonText();
     polygons.add(polygon);
-    nextToken = getNextCloserOrComma(tokenizer);
-    while (nextToken.equals(",")) {
-      polygon = readPolygonText(tokenizer);
+    nextToken = getNextCloserOrComma();
+    while (nextToken.equals(COMMA)) {
+      polygon = readPolygonText();
       polygons.add(polygon);
-      nextToken = getNextCloserOrComma(tokenizer);
+      nextToken = getNextCloserOrComma();
     }
     Polygon[] array = new Polygon[polygons.size()];
     return geometryFactory.createMultiPolygon((Polygon[]) polygons.toArray(array));
@@ -531,22 +617,23 @@ public class WKTReader {
    *      token was encountered
    *@throws  IOException     if an I/O error occurs
    */
-  private GeometryCollection readGeometryCollectionText(StreamTokenizer tokenizer) throws IOException, ParseException {
-    String nextToken = getNextEmptyOrOpener(tokenizer);
-    if (nextToken.equals("EMPTY")) {
+  private GeometryCollection readGeometryCollectionText() throws IOException, ParseException {
+    String nextToken = getNextEmptyOrOpener();
+    if (nextToken.equals(EMPTY)) {
       return geometryFactory.createGeometryCollection(new Geometry[]{});
     }
     ArrayList geometries = new ArrayList();
-    Geometry geometry = readGeometryTaggedText(tokenizer);
+    Geometry geometry = readGeometryTaggedText();
     geometries.add(geometry);
-    nextToken = getNextCloserOrComma(tokenizer);
-    while (nextToken.equals(",")) {
-      geometry = readGeometryTaggedText(tokenizer);
+    nextToken = getNextCloserOrComma();
+    while (nextToken.equals(COMMA)) {
+      geometry = readGeometryTaggedText();
       geometries.add(geometry);
-      nextToken = getNextCloserOrComma(tokenizer);
+      nextToken = getNextCloserOrComma();
     }
     Geometry[] array = new Geometry[geometries.size()];
     return geometryFactory.createGeometryCollection((Geometry[]) geometries.toArray(array));
   }
+
 }
 
