@@ -1,5 +1,3 @@
-
-
 /*
  * The JTS Topology Suite is a collection of Java classes that
  * implement the fundamental operations required to validate a given
@@ -42,25 +40,76 @@ import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 
 /**
- * Outputs the textual representation of a {@link Geometry}.
+ * Outputs the Well-Known Text representation of a {@link Geometry}.
+ * The Well-known Text format is defined in the
+ * <A HREF="http://www.opengis.org/techno/specs.htm">
+ * OGC Simple Features Specification for SQL</A>.
  * <p>
  * The <code>WKTWriter</code> outputs coordinates rounded to the precision
  * model. No more than the maximum number of necessary decimal places will be
  * output.
  * <p>
- * The Well-known Text format is defined in the <A
- * HREF="http://www.opengis.org/techno/specs.htm">OpenGIS Simple Features
- * Specification for SQL</A>.
- * <p>
- * A non-standard "LINEARRING" tag is used for LinearRings. The WKT spec does
- * not define a special tag for LinearRings. The standard tag to use is
- * "LINESTRING".
+ * A non-standard <code>LINEARRING</code> tag is used for LinearRings.
+ * The SFS WKT spec does not define a special tag for <code>LinearRing</code>s.
+ * Under it, rings are output using <code>LINESTRING</code>.
  *
- * @version 1.6
+ * @version 1.7
+ * @see WKTReader
  */
-public class WKTWriter {
+public class WKTWriter
+{
+  /**
+   * Generates the WKT for a <code>Point</code>.
+   *
+   * @param p0 the point coordinate
+   *
+   * @return the WKT
+   */
+  public static String toPoint(Coordinate p0)
+  {
+    return "POINT ( " + p0.x + " " + p0.y  + " )";
+  }
+
+  /**
+   * Generates the WKT for a N-point <code>LineString</code>.
+   *
+   * @param seq the sequence to outpout
+   *
+   * @return the WKT
+   */
+  public static String toLineString(CoordinateSequence seq)
+  {
+    StringBuffer buf = new StringBuffer();
+    buf.append("LINESTRING ");
+    if (seq.size() == 0)
+      buf.append(" EMPTY");
+    else {
+      buf.append("(");
+      for (int i = 0; i < seq.size(); i++) {
+        if (i > 0)
+          buf.append(", ");
+        buf.append(seq.getX(i) + " " + seq.getY(i));
+      }
+      buf.append(")");
+    }
+    return buf.toString();
+  }
+
+  /**
+   * Generates the WKT for a 2-point <code>LineString</code>.
+   *
+   * @param p0 the first coordinate
+   * @param p1 the second coordinate
+   *
+   * @return the WKT
+   */
+  public static String toLineString(Coordinate p0, Coordinate p1)
+  {
+    return "LINESTRING ( " + p0.x + " " + p0.y + ", " + p1.x + " " + p1.y + " )";
+  }
 
   private static int INDENT = 2;
+
   /**
    *  Creates the <code>DecimalFormat</code> used to write <code>double</code>s
    *  with a sufficient number of decimal places.
@@ -77,8 +126,8 @@ public class WKTWriter {
     // specify decimal separator explicitly to avoid problems in other locales
     DecimalFormatSymbols symbols = new DecimalFormatSymbols();
     symbols.setDecimalSeparator('.');
-    return new DecimalFormat("#" + (decimalPlaces > 0 ? "." : "")
-         + stringOfChar('#', decimalPlaces), symbols);
+    return new DecimalFormat("0" + (decimalPlaces > 0 ? "." : "")
+                 +  stringOfChar('#', decimalPlaces), symbols);
   }
 
   /**
@@ -362,25 +411,34 @@ public class WKTWriter {
     }
     else {
       writer.write("(");
-      appendCoordinate(coordinate, writer, precisionModel);
+      appendCoordinate(coordinate, writer);
       writer.write(")");
     }
   }
 
+  /**
+   *  Appends the i'th coordinate from the sequence to the writer
+   *
+   *@param  seq      the <code>CoordinateSequence</code> to process
+   * @param i the index of the coordinate to write
+   *@param  writer          the output writer to append to
+   */
+  private void appendCoordinate(CoordinateSequence seq, int i, Writer writer)
+    throws IOException
+  {
+    writer.write(writeNumber(seq.getX(i)) + " " + writeNumber(seq.getY(i)));
+  }
+
   /**
    *  Converts a <code>Coordinate</code> to &lt;Point&gt; format, then appends
    *  it to the writer.
    *
    *@param  coordinate      the <code>Coordinate</code> to process
    *@param  writer          the output writer to append to
-   *@param  precisionModel  the <code>PrecisionModel</code> to use to convert
-   *      from a precise coordinate to an external coordinate
    */
-  private void appendCoordinate(Coordinate coordinate, Writer writer, PrecisionModel precisionModel)
+  private void appendCoordinate(Coordinate coordinate, Writer writer)
     throws IOException
   {
-    //Coordinate externalCoordinate = new Coordinate();
-    //precisionModel.toExternal(coordinate, externalCoordinate);
     writer.write(writeNumber(coordinate.x) + " " + writeNumber(coordinate.y));
   }
 
@@ -396,6 +454,33 @@ public class WKTWriter {
     return formatter.format(d);
   }
 
+  /**
+   *  Converts a <code>LineString</code> to &lt;LineString Text&gt; format, then
+   *  appends it to the writer.
+   *
+   *@param  lineString  the <code>LineString</code> to process
+   *@param  writer      the output writer to append to
+   */
+  private void appendSequenceText(CoordinateSequence seq, int level, boolean doIndent, Writer writer)
+    throws IOException
+  {
+    if (seq.size() == 0) {
+      writer.write("EMPTY");
+    }
+    else {
+      if (doIndent) indent(level, writer);
+      writer.write("(");
+      for (int i = 0; i < seq.size(); i++) {
+        if (i > 0) {
+          writer.write(", ");
+          if (i % 10 == 0) indent(level + 2, writer);
+        }
+        appendCoordinate(seq, i, writer);
+      }
+      writer.write(")");
+    }
+  }
+
   /**
    *  Converts a <code>LineString</code> to &lt;LineString Text&gt; format, then
    *  appends it to the writer.
@@ -417,7 +502,7 @@ public class WKTWriter {
           writer.write(", ");
           if (i % 10 == 0) indent(level + 2, writer);
         }
-        appendCoordinate(lineString.getCoordinateN(i), writer, lineString.getPrecisionModel());
+        appendCoordinate(lineString.getCoordinateN(i), writer);
       }
       writer.write(")");
     }
@@ -467,8 +552,7 @@ public class WKTWriter {
         if (i > 0) {
           writer.write(", ");
         }
-        appendCoordinate(((Point) multiPoint.getGeometryN(i)).getCoordinate(), writer,
-            multiPoint.getPrecisionModel());
+        appendCoordinate(((Point) multiPoint.getGeometryN(i)).getCoordinate(), writer);
       }
       writer.write(")");
     }
