@@ -46,7 +46,7 @@ import com.vividsolutions.jts.operation.GeometryGraphOperation;
  * Computes the overlay of two {@link Geometry}s.  The overlay
  * can be used to determine any boolean combination of the geometries.
  *
- * @version 1.6
+ * @version 1.7
  */
 public class OverlayOp
   extends GeometryGraphOperation
@@ -75,7 +75,7 @@ public class OverlayOp
   }
 
   /**
-   * This method will handle arguments of Location.NULL correctly
+   * This method will handle arguments of Location.NONE correctly
    *
    * @return true if the locations correspond to the opCode
    */
@@ -206,11 +206,12 @@ public class OverlayOp
    */
   protected void insertUniqueEdge(Edge e)
   {
-//Debug.println(e);
-    int foundIndex = edgeList.findEdgeIndex(e);
+//<FIX> MD 8 Oct 03  speed up identical edge lookup
+    // fast lookup
+    Edge existingEdge = edgeList.findEqualEdge(e);
+
     // If an identical edge already exists, simply update its label
-    if (foundIndex >= 0) {
-      Edge existingEdge = (Edge) edgeList.get(foundIndex);
+    if (existingEdge != null) {
       Label existingLabel = existingEdge.getLabel();
 
       Label labelToMerge = e.getLabel();
