@@ -38,14 +38,14 @@ import java.util.*;
 import com.vividsolutions.jts.algorithm.*;
 import com.vividsolutions.jts.geomgraph.*;
 import com.vividsolutions.jts.geom.*;
-import com.vividsolutions.jts.operation.GeometryGraphOperation;
 import com.vividsolutions.jts.util.*;
 
 /**
  * Implements the algorithsm required to compute the <code>isValid()</code> method
  * for {@link Geometry}s.
+ * See the documentation for the various geometry types for a specification of validity.
  *
- * @version 1.6
+ * @version 1.7
  */
 public class IsValidOp
 {
@@ -91,6 +91,11 @@ public class IsValidOp
   }
 
   private Geometry parentGeometry;  // the base Geometry to be validated
+  /**
+   * If the following condition is TRUE JTS will validate inverted shells and exverted holes
+   * (the ESRI SDE model)
+   */
+  private boolean isSelfTouchingRingFormingHoleValid = false;
   private boolean isChecked = false;
   private TopologyValidationError validErr;
 
@@ -99,6 +104,35 @@ public class IsValidOp
     this.parentGeometry = parentGeometry;
   }
 
+  /**
+   * Sets whether polygons using <b>Self-Touching Rings</b> to form
+   * holes are reported as valid.
+   * If this flag is set, the following Self-Touching conditions
+   * are treated as being valid:
+   * <ul>
+   * <li>the shell ring self-touches to create a hole touching the shell
+   * <li>a hole ring self-touches to create two holes touching at a point
+   * </ul>
+   * <p>
+   * The default (following the OGC SFS standard)
+   * is that this condition is <b>not</b> valid (<code>false</code>).
+   * <p>
+   * This does not affect whether Self-Touching Rings
+   * disconnecting the polygon interior are considered valid
+   * (these are considered to be <b>invalid</b> under the SFS, and many other
+   * spatial models as well).
+   * This includes "bow-tie" shells,
+   * which self-touch at a single point causing the interior to
+   * be disconnected,
+   * and "C-shaped" holes which self-touch at a single point causing an island to be formed.
+   *
+   * @param isValid states whether geometry with this condition is valid
+   */
+  public void setSelfTouchingRingFormingHoleValid(boolean isValid)
+  {
+    isSelfTouchingRingFormingHoleValid = isValid;
+  }
+
   public boolean isValid()
   {
     checkValid(parentGeometry);
@@ -115,7 +149,10 @@ public class IsValidOp
   {
     if (isChecked) return;
     validErr = null;
+
+    // empty geometries are always valid!
     if (g.isEmpty()) return;
+
     if (g instanceof Point)                   checkValid((Point) g);
     else if (g instanceof MultiPoint)         checkValid((MultiPoint) g);
                         // LineString also handles LinearRings
@@ -159,6 +196,9 @@ public class IsValidOp
   {
     checkInvalidCoordinates(g.getCoordinates());
     if (validErr != null) return;
+    checkClosedRing(g);
+    if (validErr != null) return;
+
     GeometryGraph graph = new GeometryGraph(0, g);
     checkTooFewPoints(graph);
     if (validErr != null) return;
@@ -175,6 +215,8 @@ public class IsValidOp
   {
     checkInvalidCoordinates(g);
     if (validErr != null) return;
+    checkClosedRings(g);
+    if (validErr != null) return;
 
     GeometryGraph graph = new GeometryGraph(0, g);
 
@@ -182,8 +224,11 @@ public class IsValidOp
     if (validErr != null) return;
     checkConsistentArea(graph);
     if (validErr != null) return;
-    checkNoSelfIntersectingRings(graph);
-    if (validErr != null) return;
+
+    if (! isSelfTouchingRingFormingHoleValid) {
+      checkNoSelfIntersectingRings(graph);
+      if (validErr != null) return;
+    }
     checkHolesInShell(g, graph);
     if (validErr != null) return;
     //SLOWcheckHolesNotNested(g);
@@ -191,12 +236,15 @@ public class IsValidOp
     if (validErr != null) return;
     checkConnectedInteriors(graph);
   }
+
   private void checkValid(MultiPolygon g)
   {
     for (int i = 0; i < g.getNumGeometries(); i++) {
       Polygon p = (Polygon) g.getGeometryN(i);
       checkInvalidCoordinates(p);
       if (validErr != null) return;
+      checkClosedRings(p);
+      if (validErr != null) return;
     }
 
     GeometryGraph graph = new GeometryGraph(0, g);
@@ -205,9 +253,10 @@ public class IsValidOp
     if (validErr != null) return;
     checkConsistentArea(graph);
     if (validErr != null) return;
-    checkNoSelfIntersectingRings(graph);
-    if (validErr != null) return;
-
+    if (! isSelfTouchingRingFormingHoleValid) {
+      checkNoSelfIntersectingRings(graph);
+      if (validErr != null) return;
+    }
     for (int i = 0; i < g.getNumGeometries(); i++) {
       Polygon p = (Polygon) g.getGeometryN(i);
       checkHolesInShell(p, graph);
@@ -240,10 +289,10 @@ public class IsValidOp
                           TopologyValidationError.INVALID_COORDINATE,
                           coords[i]);
         return;
-
       }
     }
   }
+
   private void checkInvalidCoordinates(Polygon poly)
   {
     checkInvalidCoordinates(poly.getExteriorRing().getCoordinates());
@@ -254,6 +303,24 @@ public class IsValidOp
     }
   }
 
+  private void checkClosedRings(Polygon poly)
+  {
+    checkClosedRing((LinearRing) poly.getExteriorRing());
+    if (validErr != null) return;
+    for (int i = 0; i < poly.getNumInteriorRing(); i++) {
+      checkClosedRing((LinearRing) poly.getInteriorRingN(i));
+      if (validErr != null) return;
+    }
+  }
+
+  private void checkClosedRing(LinearRing ring)
+  {
+    if (! ring.isClosed() )
+      validErr = new TopologyValidationError(
+                        TopologyValidationError.RING_NOT_CLOSED,
+                        ring.getCoordinateN(0));
+  }
+
   private void checkTooFewPoints(GeometryGraph graph)
   {
     if (graph.hasTooFewPoints()) {
@@ -264,6 +331,14 @@ public class IsValidOp
     }
   }
 
+  /**
+   * Checks that the arrangement of edges in a polygonal geometry graph
+   * forms a consistent area.
+   *
+   * @param graph
+   *
+   * @see ConsistentAreaTester
+   */
   private void checkConsistentArea(GeometryGraph graph)
   {
     ConsistentAreaTester cat = new ConsistentAreaTester(graph);
@@ -281,22 +356,29 @@ public class IsValidOp
     }
   }
 
+  /**
+   * Check that there is no ring which self-intersects (except of course at its endpoints).
+   * This is required by OGC topology rules (but not by other models
+   * such as ESRI SDE, which allow inverted shells and exverted holes).
+   *
+   * @param graph the topology graph of the geometry
+   */
   private void checkNoSelfIntersectingRings(GeometryGraph graph)
   {
     for (Iterator i = graph.getEdgeIterator(); i.hasNext(); ) {
       Edge e = (Edge) i.next();
-      checkSelfIntersectingRing(e.getEdgeIntersectionList());
+      checkNoSelfIntersectingRing(e.getEdgeIntersectionList());
       if (validErr != null)
         return;
     }
   }
 
   /**
-   * check that a ring does not self-intersect, except at its endpoints.
+   * Check that a ring does not self-intersect, except at its endpoints.
    * Algorithm is to count the number of times each node along edge occurs.
    * If any occur more than once, that must be a self-intersection.
    */
-  private void checkSelfIntersectingRing(EdgeIntersectionList eiList)
+  private void checkNoSelfIntersectingRing(EdgeIntersectionList eiList)
   {
     Set nodeSet = new TreeSet();
     boolean isFirst = true;
@@ -318,25 +400,14 @@ public class IsValidOp
     }
   }
 
-  /* NO LONGER NEEDED AS OF JTS Ver 1.2
-  private void checkNoRepeatedPoint(Geometry g)
-  {
-    RepeatedPointTester rpt = new RepeatedPointTester();
-    if (rpt.hasRepeatedPoint(g)) {
-      validErr = new TopologyValidationError(
-                        TopologyValidationError.REPEATED_POINT,
-                        rpt.getCoordinate());
-    }
-  }
-  */
-
   /**
    * Tests that each hole is inside the polygon shell.
    * This routine assumes that the holes have previously been tested
-   * to ensure that all vertices lie on the shell or inside it.
-   * A simple test of a single point in the hole can be used,
-   * provide the point is chosen such that it does not lie on the
-   * boundary of the shell.
+   * to ensure that all vertices lie on the shell oon the same side of it
+   * (i.e that the hole rings do not cross the shell ring).
+   * In other words, this test is only correct if the ConsistentArea test is passed first.
+   * Given this, a simple point-in-polygon test of a single point in the hole can be used,
+   * provided the point is chosen such that it does not lie on the shell.
    *
    * @param p the polygon to be tested for hole inclusion
    * @param graph a GeometryGraph incorporating the polygon
@@ -344,7 +415,6 @@ public class IsValidOp
   private void checkHolesInShell(Polygon p, GeometryGraph graph)
   {
     LinearRing shell = (LinearRing) p.getExteriorRing();
-    Coordinate[] shellPts = shell.getCoordinates();
 
     //PointInRing pir = new SimplePointInRing(shell);
     //PointInRing pir = new SIRtreePointInRing(shell);
@@ -354,7 +424,12 @@ public class IsValidOp
 
       LinearRing hole = (LinearRing) p.getInteriorRingN(i);
       Coordinate holePt = findPtNotNode(hole.getCoordinates(), shell, graph);
-      Assert.isTrue(holePt != null, "Unable to find a hole point not a vertex of the shell");
+      /**
+       * If no non-node hole vertex can be found, the hole must
+       * split the polygon into disconnected interiors.
+       * This will be caught by a subsequent check.
+       */
+      if (holePt == null) return;
 
       boolean outside = ! pir.isInside(holePt);
       if ( outside ) {
